% Checking the model with DYNARE

% Declare the endogenous variables
var y c k H i g A V lambda;

% Variable definitions
% y = output
% c = consumption
% i = investment
% k = capital
% H = hours worked
% i = investment
% g = government spending
% A = neutral technology
% V = investment-specific technology
% lambda = shadow value

% Declare the exogenous variables
varexo epsA epsg epsV;

% Declare the parameters

parameters eta delta beta theta rhoA rhog rhoV sigA sigg sigV kappa g_y nu q k_y si sc phi;
eta=1.00055;
delta=0.021;
beta=0.99;
theta=0.36;
rhoA=0.95;
rhog=0.8;
rhoV=0.95;
g_y=0.2;
kappa=(eta/beta)-1+delta;
sigA=0.0066;
sigg=0.0066;
sigV=0.0066;
nu=0.7;
q=25;
k_y=theta*beta/(eta-beta*(1-delta));
si=(eta-1+delta)*k_y;
sc=1-g_y-si;
phi=(eta-beta+beta*delta)/eta;

model(linear);
lambda-V+q*eta*k-q*eta*k(-1)=lambda(+1)+phi*rhoA*A-phi*(1-theta)*k+phi*(1-theta)*H(+1)-(1-delta)*(beta/eta)*rhoV*V+beta*q*eta*k(+1)-beta*q*eta*k;
y = A + theta*k(-1)+(1-theta)*H;
y = sc*c+eta*k_y*k-(1-delta)*k_y*k(-1)+k_y*(1-delta-eta)*V+g_y*g;
0=lambda+A+theta*k(-1)-theta*H;
(beta*nu/eta)/(1-nu/eta)*c(+1)-((1+beta*(nu/eta)^2)/(1-nu/eta))*c+((nu/eta)/(1-nu/eta))*c(-1)=(1-beta*nu/eta)*lambda;
eta*k=(1-delta)*k(-1)+(eta-1+delta)*i+(eta-1+delta)*V;
A=rhoA*A(-1)+epsA;
g=rhog*g(-1)+epsg;
V=rhoV*V(-1)+epsV;
end;

% Declare the variances of the shock process
shocks;
var epsA; stderr sigA;
var epsg; stderr sigg;
var epsV; stderr sigV;
end;

check;

stoch_simul(linear,irf=40) y i H c A g V;
