% -------------------------------------------------------------------------------------------------------------------------
% THIS PROGRAM CONTAINS THE MATLAB CODES FOR SOLVING THE MODEL HABIT FOR THE PAPER
% "COMPARING MODELS OF MACROECONOMIC FLUCTUATIONS: HOW BIG ARE THE DIFFERENCES?" BY ANDRA GHENT (2008)
% -------------------------------------------------------------------------------------------------------------------------

function [GAMMAXX,GAMMAYX,GAMMAYY]=HABITGAMMAs(x,X,Y);

% Input is a vector of parameters and the number of period to run the
% impulse response function for
% Output is the impulse response functions for neutral technology shock

%% Other parameters
y0=X(1,14);%/x(2,1);
i0=X(1,15);%/x(2,1);
h0=X(1,16);%/x(2,1);
c0=X(1,17);%/x(2,1);

[T,n]=size(Y);
[T,k]=size(X);

% Parametrization 
beta=0.99;  
theta=x(1,:);  
eta=x(2,:);
delta=x(3,:);                     
rhoA=x(4,:);
rhog=x(5,:);
rhoV=x(6,:);
g_y=x(7,:);
sigA=x(8,:);
sigg=x(9,:);
sigV=x(10,:);
sigmeas=x(11,:);
nu=x(12,:);
q=x(13,:);
                      
psi = (eta-beta*(1-delta))/eta;

si=theta*(eta-1+delta)/(eta/beta-1+delta);      % investment share of output
sc=(1-si-g_y);                                  % consumption's share of output
k_y=theta*beta/(eta-beta*(1-delta));            % steady state capital to output ratio
    
a1=(beta*nu/eta)/(1-nu/eta);
a2=-((1+beta*(nu^2/eta^2))/(1-nu/eta));
a3=(nu/eta)/(1-nu/eta);
a4=(1-beta*nu/eta);

a5=sc/(1-theta);
a6=eta*k_y/(1-theta);
a7=-((1-delta)*k_y+theta)/(1-theta);
a8=-1/(1-theta);
a9=g_y/(1-theta);
a10=k_y*(1-delta-eta)/(1-theta);

a11=theta*a5;
a12=theta*a6;
a13=theta*a7-theta;
a14=theta*a8-1;
a15=theta*a9;
a16=theta*a10;

b1=a1;
b2=a2-a4*a11;
b3=a3;
b4=a4*a12;
b5=a4*a13;
b6=a4*a14;
b7=a4*a15;
b8=a4*a16;

b9=a11+(1-theta)*psi*a5;
b10=a12+a6*(1-theta)*psi+beta*q*eta;
b11=a13+(1-theta)*psi*a7-(1-theta)*psi-beta*q*eta-a12-eta*q;
b12=a11;
b13=a13-q*eta;
b14=a14-rhoA*(a14+psi+(1-theta)*psi*a8);
b15=a15*(1-rhog)-psi*(1-theta)*rhog*a9;
b16=a16-1-a16*rhoV-psi*(1-theta)*rhoV*a10+(1-delta)*beta*rhoV/eta;


A=[b2 0 b1 0; 0 b11 b9 b10; 1 0 0 0; 0 1 0 0];
B=[-b3 b5 0 b4; 0 b13 b12 0; 0 0 1 0; 0 0 0 1];
C=[b6 b7 b8; b14 b15 b16; 0 0 0; 0 0 0];

invA=inv(A);
D=invA*B;
F=invA*C;

[GinverseBG,H]=eigs(D);        % command eigs sorts the eigenvalues by largest to smallest

H;

% A necessary condition for a unique solution to exist, by Blanchard-Kahn
% (1980) Proposition 1, is that the number of non-predetermined vaiables is
% equal to the number of eigenvalues of D outside the unit circle
% here, this means we need 2 eigenvalues greater than 1

% We're all good here!! Finally!

lambda1H=H(4,4);
lambda2H=H(3,3);
lambda3H=H(2,2);
lambda4H=H(1,1);

Ginv=zeros(4,4);
Ginv(:,1)=GinverseBG(:,4);
Ginv(:,2)=GinverseBG(:,3);
Ginv(:,3)=GinverseBG(:,2);
Ginv(:,4)=GinverseBG(:,1);
G=inv(Ginv);
G11=G(1,1);
G12=G(1,2);
G13=G(1,3);
G14=G(1,4);
G21=G(2,1);
G22=G(2,2);
G23=G(2,3);
G24=G(2,4);
G31=G(3,1);
G32=G(3,2);
G33=G(3,3);
G34=G(3,4);
G41=G(4,1);
G42=G(4,2);
G43=G(4,3);
G44=G(4,4);

J=G*F;
J11=J(1,1);
J12=J(1,2);
J13=J(1,3);
J21=J(2,1);
J22=J(2,2);
J23=J(2,3);
J31=J(3,1);
J32=J(3,2);
J33=J(3,3);
J41=J(4,1);
J42=J(4,2);
J43=J(4,3);

d1=(1-G43*G34/(G44*G33))^(-1)*(G43*G31/(G44*G33)-G41/G44);
d2=(1-G43*G34/(G44*G33))^(-1)*(G43*G32/(G44*G33)-G42/G44);
d3=(1-G43*G34/(G44*G33))^(-1)*(J41/(G44*(rhoA-lambda4H))-G43*J31/(G44*(rhoA-lambda3H)*G33));
d4=(1-G43*G34/(G44*G33))^(-1)*(J42/(G44*(rhog-lambda4H))-G43*J32/(G44*(rhog-lambda3H)*G33));
d5=(1-G43*G34/(G44*G33))^(-1)*(J43/(G44*(rhoV-lambda4H))-G43*J33/(G44*(rhoV-lambda3H)*G33));

d6=-(G31/G33+G34*d1/G33);
d7=-(G32/G33+G34*d2/G33);
d8=J31/((rhoA-lambda3H)*G33)-G34*d3/G33;
d9=J32/((rhog-lambda3H)*G33)-G34*d4/G33;
d10=J33/((rhoV-lambda3H)*G33)-G34*d5/G33;

j1=G11+G13*d6+G14*d1;
j2=G12+G13*d7+G14*d2;
j3=lambda1H*j1;
j4=lambda1H*j2;
j5=(lambda1H-rhoA)*(G13*d8+G14*d3)+J11;
j6=(lambda1H-rhog)*(G13*d9+G14*d4)+J12;
j7=(lambda1H-rhoV)*(G13*d10+G14*d5)+J13;

j8=G21+G23*d6+G24*d1;
j9=G22+G23*d7+G24*d2;
j10=lambda2H*j8;
j11=lambda2H*j9;
j12=(lambda2H-rhoA)*(G23*d8+G24*d3)+J21;
j13=(lambda2H-rhog)*(G23*d9+G24*d4)+J22;
j14=(lambda2H-rhoV)*(G23*d10+G24*d5)+J23;

K1=[j1 j2 0 0 0; j8 j9 0 0 0; 0 0 1 0 0; 0 0 0 1 0; 0 0 0 0 1];
K2=[j3 j4 j5 j6 j7; j10 j11 j12 j13 j14; 0 0 rhoA 0 0; 0 0 0 rhog 0; 0 0 0 0 rhoV];
K3=[0 0 0; 0 0 0; 1 0 0; 0 1 0; 0 0 1];
PI=inv(K1)*K2;
W=inv(K1)*K3;

PI11=PI(1,1);
PI12=PI(1,2);
PI13=PI(1,3);
PI14=PI(1,4);
PI15=PI(1,5);
PI21=PI(2,1);
PI22=PI(2,2);
PI23=PI(2,3);
PI24=PI(2,4);
PI25=PI(2,5);
PI31=PI(3,1);
PI32=PI(3,2);
PI33=PI(3,3);
PI34=PI(3,4);
PI35=PI(3,5);

m1=a5*PI11+a6*PI21;
m2=a5*PI12+a6*PI22+a7;
m3=a5*PI13+a6*PI23+a8;
m4=a5*PI14+a6*PI24+a9;
m5=a5*PI15+a6*PI25+a10;

m6=(1-theta)*m1;
m7=theta+(1-theta)*m2;
m8=(1-theta)*m3+1;
m9=m4*(1-theta);
m10=m5*(1-theta);

m11=eta*PI21/(eta-1+delta);
m12=eta*PI22/(eta-1+delta)-(1-delta)/(eta-1+delta);
m13=eta*PI23/(eta-1+delta);
m14=eta*PI24/(eta-1+delta);
m15=eta*PI25/(eta-1+delta);

U=[m6 m7 m8 m9 m10; m11 m12 m13 m14 m15;m1 m2 m3 m4 m5; PI11 PI12 PI13 PI14 PI15];


% Now solve for omegass
sigepsilon=[sigA^2 0 0; 0 sigg^2 0; 0 0 sigV^2];

CC=W*sigepsilon*W';
OMEGAss=dlyap(PI,CC);

VARMEAS=[sigmeas^2 0 0 0; 0 0 0 0; 0 0 0 0; 0 0 0 0];

SIGyy=U*OMEGAss*U'+VARMEAS;

n=4;            % number of variables 
p=4;            % number of lags in the VAR
q=n*p+1;        % total number of variables in the VAR

SIGyx=zeros(n,q);
for k=1:p
    SIGyx(:,(k-1)*n+2:k*n+1)=U*PI^k*OMEGAss*U';
end

% Creating a matrix that will keep take into account the trend in E(Y*'*Y*)
etasq=0;
etaonh=0;
sumTs=0;
for j=1:T
    etasq=etasq+eta^(2*(p+j));
    etaonh=etaonh+eta^(p+j);
    sumTs=sumTs+1;
end
SIGyytrend=etasq*ones(n,n);
SIGyytrend(1,3)=etaonh;
SIGyytrend(2,3)=etaonh;
SIGyytrend(3,1)=etaonh;
SIGyytrend(3,2)=etaonh;
SIGyytrend(3,3)=sumTs;
SIGyytrend(3,4)=etaonh;
SIGyytrend(4,3)=etaonh;
SIGyytrend;

% Creating a matrix that will take into account initial conditions
SIGyyinit=zeros(n,n);
SIGyyinit(1,1)=y0^2;
SIGyyinit(1,2)=y0*i0;
SIGyyinit(1,3)=y0*h0;
SIGyyinit(1,4)=y0*c0;
SIGyyinit(2,1)=i0*y0;
SIGyyinit(2,2)=i0^2;
SIGyyinit(2,3)=i0*h0;
SIGyyinit(2,4)=i0*c0;
SIGyyinit(3,1)=h0*y0;
SIGyyinit(3,2)=h0*i0;
SIGyyinit(3,3)=h0^2;
SIGyyinit(3,4)=h0*c0;
SIGyyinit(4,1)=c0*y0;
SIGyyinit(4,2)=c0*i0;
SIGyyinit(4,3)=c0*h0;
SIGyyinit(4,4)=c0^2;

GAMMAYY=T^(-1)*SIGyytrend.*(SIGyy+SIGyyinit);

SIGyx=zeros(n,q);
for kk=1:p
    SIGyx(:,(kk-1)*n+2:kk*n+1)=U*PI^kk*OMEGAss*U';
end
SIGyxtrend=zeros(n,q);
SIGyxtrend(1,1)=etaonh;
SIGyxtrend(2,1)=etaonh;
SIGyxtrend(3,1)=sumTs;
SIGyxtrend(4,1)=etaonh;
SIGyxtrend(1,2)=etasq/eta;
SIGyxtrend(1,3)=etasq/eta;
SIGyxtrend(1,4)=etaonh;
SIGyxtrend(1,5)=etasq/eta;
SIGyxtrend(1,6)=etasq/eta^2;
SIGyxtrend(1,7)=etasq/eta^2;
SIGyxtrend(1,8)=etaonh;
SIGyxtrend(1,9)=etasq/eta^2;
SIGyxtrend(1,10)=etasq/eta^3;
SIGyxtrend(1,11)=etasq/eta^3;
SIGyxtrend(1,12)=etaonh;
SIGyxtrend(1,13)=etasq/eta^3;
SIGyxtrend(1,14)=etasq/eta^4;
SIGyxtrend(1,15)=etasq/eta^4;
SIGyxtrend(1,16)=etaonh;
SIGyxtrend(1,17)=etasq/eta^4;
SIGyxtrend(2,2:17)=SIGyxtrend(1,2:17);
SIGyxtrend(3,2)=etaonh/eta;
SIGyxtrend(3,3)=etaonh/eta;
SIGyxtrend(3,4)=sumTs;
SIGyxtrend(3,5)=etaonh/eta;
SIGyxtrend(3,6)=etaonh/eta^2;
SIGyxtrend(3,7)=etaonh/eta^2;
SIGyxtrend(3,8)=sumTs;
SIGyxtrend(3,9)=etaonh/eta^2;
SIGyxtrend(3,10)=etaonh/eta^3;
SIGyxtrend(3,11)=etaonh/eta^3;
SIGyxtrend(3,12)=sumTs;
SIGyxtrend(3,13)=etaonh/eta^3;
SIGyxtrend(3,14)=etaonh/eta^4;
SIGyxtrend(3,15)=etaonh/eta^4;
SIGyxtrend(3,16)=sumTs;
SIGyxtrend(3,17)=etaonh/eta^4;
SIGyxtrend(4,2:17)=SIGyxtrend(1,2:17);
SIGyxtrend;

SIGyxinit=zeros(n,q);
SIGyxinit(1:4,1)=[y0 i0 h0 c0];
SIGyxinit(1:4,2:5)=[y0^2 y0*i0 y0*h0 y0*c0;y0*i0 i0^2 i0*h0 i0*c0;...
    h0*y0 h0*i0 h0^2 h0*c0; c0*y0 c0*i0 c0*h0 c0^2];
SIGyxinit(1:4,6:9)=SIGyxinit(:,2:5);
SIGyxinit(:,10:13)=SIGyxinit(:,2:5);
SIGyxinit(:,14:17)=SIGyxinit(:,2:5);
SIGyxtrend.*SIGyxinit;

GAMMAYX=T^(-1)*SIGyxtrend.*(SIGyx + SIGyxinit);
    

SIGxx=zeros(q,q);
for kk=1:p
    SIGxx(2+(kk-1)*n:1+kk*n,2+(kk-1)*n:1+kk*n)=SIGyy;
end
SIGxx(2:5,6:9)=U*PI*OMEGAss*U';
SIGxx(2:5,10:13)=U*PI^2*OMEGAss*U';
SIGxx(2:5,14:17)=U*PI^3*OMEGAss*U';
SIGxx(6:9,2:5)=U*OMEGAss*PI'*U';
SIGxx(6:9,10:13)=U*PI*OMEGAss*U';
SIGxx(6:9,14:17)=U*PI^2*OMEGAss*U';
SIGxx(10:13,2:5)=U*OMEGAss*(PI^2)'*U';
SIGxx(10:13,6:9)=U*OMEGAss*PI'*U';
SIGxx(10:13,14:17)=U*PI*OMEGAss*U';
SIGxx(14:17,2:5)=U*OMEGAss*(PI^3)'*U';
SIGxx(14:17,6:9)=U*OMEGAss*(PI^2)'*U';
SIGxx(14:17,10:13)=U*OMEGAss*PI'*U';

SIGxxtrend=zeros(q,q);
SIGxxtrend(1,1)=sumTs;
SIGxxtrend(1,2)=etaonh/eta;
SIGxxtrend(1,3)=etaonh/eta;
SIGxxtrend(1,4)=sumTs;
SIGxxtrend(1,5)=etaonh/eta;
SIGxxtrend(1,6)=etaonh/eta^2;
SIGxxtrend(1,7)=etaonh/eta^2;
SIGxxtrend(1,8)=sumTs;
SIGxxtrend(1,9)=etaonh/eta^2;
SIGxxtrend(1,10)=etaonh/eta^3;
SIGxxtrend(1,11)=etaonh/eta^3;
SIGxxtrend(1,12)=sumTs;
SIGxxtrend(1,13)=etaonh/eta^3;
SIGxxtrend(1,14)=etaonh/eta^4;
SIGxxtrend(1,15)=etaonh/eta^4;
SIGxxtrend(1,16)=sumTs;
SIGxxtrend(1,17)=etaonh/eta^4;
SIGxxtrend(2,1)=etaonh/eta;
SIGxxtrend(2,2)=etasq/eta^2;
SIGxxtrend(2,3)=etasq/eta^2;
SIGxxtrend(2,4)=etaonh/eta;
SIGxxtrend(2,5)=etasq/eta^2;
SIGxxtrend(2,6:9)=SIGxxtrend(2,2:5)./eta;
SIGxxtrend(2,10:13)=SIGxxtrend(2,2:5)./eta^2;
SIGxxtrend(2,14:17)=SIGxxtrend(2,2:5)/eta^3;
SIGxxtrend(3,:)=SIGxxtrend(2,:);
SIGxxtrend(4,1)=sumTs;
SIGxxtrend(4,2)=etaonh/eta;
SIGxxtrend(4,3)=etaonh/eta;
SIGxxtrend(4,4)=sumTs;
SIGxxtrend(4,5)=etaonh/eta;
SIGxxtrend(4,6)=etaonh/eta^2;
SIGxxtrend(4,7)=etaonh/eta^2;
SIGxxtrend(4,8)=sumTs;
SIGxxtrend(4,9)=etaonh/eta^2;
SIGxxtrend(4,10)=etaonh/eta^3;
SIGxxtrend(4,11)=etaonh/eta^3;
SIGxxtrend(4,12)=sumTs;
SIGxxtrend(4,13)=etaonh/eta^3;
SIGxxtrend(4,14)=etaonh/eta^4;
SIGxxtrend(4,15)=etaonh/eta^4;
SIGxxtrend(4,16)=sumTs;
SIGxxtrend(4,17)=etaonh/eta^4;
SIGxxtrend(5,:)=SIGxxtrend(2,:);
SIGxxtrend(6,:)=SIGxxtrend(2,:)./eta;
SIGxxtrend(7,:)=SIGxxtrend(3,:)./eta;
SIGxxtrend(8,:)=SIGxxtrend(4,:)./eta;
SIGxxtrend(8,1)=sumTs;
SIGxxtrend(8,4)=sumTs;
SIGxxtrend(8,8)=sumTs;
SIGxxtrend(8,12)=sumTs;
SIGxxtrend(8,16)=sumTs;
SIGxxtrend(9,:)=SIGxxtrend(5,:)./eta;
SIGxxtrend(10,:)=SIGxxtrend(6,:)./eta;
SIGxxtrend(11,:)=SIGxxtrend(7,:)./eta;
SIGxxtrend(12,:)=SIGxxtrend(8,:)./eta;
SIGxxtrend(12,1)=sumTs;
SIGxxtrend(12,4)=sumTs;
SIGxxtrend(12,8)=sumTs;
SIGxxtrend(12,12)=sumTs;
SIGxxtrend(12,16)=sumTs;
SIGxxtrend(13,:)=SIGxxtrend(9,:)./eta;
SIGxxtrend(14,:)=SIGxxtrend(10,:)./eta;
SIGxxtrend(15,:)=SIGxxtrend(11,:)./eta;
SIGxxtrend(16,:)=SIGxxtrend(12,:)./eta;
SIGxxtrend(16,1)=sumTs;
SIGxxtrend(16,4)=sumTs;
SIGxxtrend(16,8)=sumTs;
SIGxxtrend(16,12)=sumTs;
SIGxxtrend(16,16)=sumTs;
SIGxxtrend(17,:)=SIGxxtrend(13,:)./eta;

iota=ones(1,4);
init=[y0;i0;h0;c0];
initsq=[y0^2 y0*i0 y0*h0 y0*c0; i0*y0 i0^2 i0*h0 i0*c0; h0*y0 h0*i0 h0^2 h0*c0; c0*y0 c0*i0 c0*h0 c0^2];
SIGxxinit=zeros(q,q);
SIGxxinit(:,1)=[1; reshape(kron(init,iota),16,1)];
SIGxxinit(1,:)=SIGxxinit(:,1)';
SIGxxinit(2:17,2:17)=[repmat(initsq,4)];

GAMMAXX=T^(-1)*SIGxxtrend.*(SIGxx+SIGxxinit);