function PHIS=HABITdrawsdetrend(X,Y,zetas,lambda);

% Inputs = X (RHS variable), Y(LHS Variable),zetas(DSGE model parameter 
% vector), lambda (hyperparameter)

% Outputs = PHIs (n*p+1)xn Coefficient matrix

% This program computes draws from the posterior distribution for the model
% RBC

    % Order of parameters in RBC priors is beta, theta, eta, delta, rhoA,
    % rhog, rhoV,g_y, sigA, sigg, sigV, sigmeas

%% Generating draws from posterior distribution of PHI
[nsim,junk]=size(zetas);
[T,k]=size(X);
[T,n]=size(Y);
PHIS=zeros(k,n,nsim);                  
PHI=zeros(k*n,1);

for ii=1:nsim    
    % Computing relevant matrices at draw
    [GAMMAXX,GAMMAYX,GAMMAYY,SIGUtilde]=postHABITdetrend(zetas(ii,:)',X,Y,lambda);
    XX=X'*X;
    PHItilde=inv(lambda*T*GAMMAXX+XX)*(lambda*T*GAMMAYX'+X'*Y);
    
    % Take the Choleski Decomposition of SIGUtilde to avoid numerical
    % problems
    eigSIGUtilde=eig(SIGUtilde);
    % Discard draws that don't lead to a positive definite SIGUtilde or
    % otherwise lead to numerical problems
    if isreal(eigSIGUtilde)&&(min(eigSIGUtilde)>0) 
        D=chol((lambda+1)*T*SIGUtilde);
        
        % Draw from the marginal posterior of Sigmau
        SIGu=iwishrnd(D'*D,(1+lambda)*T-k);
        PHItilde=reshape(PHItilde,n*k,1);
        SIGu=double(kron(SIGu,inv(lambda*T*GAMMAXX+X'*X)));
        HABITeigsSIGu=min(eig(SIGu));
        HABITdetSIGu=det(SIGu);
        if HABITeigsSIGu>1e-10
            D=chol(SIGu);
            SIGu=D'*D;   
            % Now draw from the posterior of PHI
            PHI=PHItilde+norm_rnd(SIGu);
        else
            PHI=PHItilde;
        end
    else
        PHI=PHI;
    end
        
    % Save the draw
    PHIS(:,:,ii)=reshape(PHI,k,n);
end
    
    