% -------------------------------------------------------------------------------------------------------------------------
% THIS PROGRAM CONTAINS THE MATLAB CODES FOR SOLVING THE MODEL HABIT FOR THE PAPER
% "COMPARING MODELS OF MACROECONOMIC FLUCTUATIONS: HOW BIG ARE THE DIFFERENCES?" BY ANDRA GHENT (2008)
% -------------------------------------------------------------------------------------------------------------------------

function [IRFSneutral,IRFSgovt,IRFSishock]=HABITprior(params,n);

% Input is a vector of parameters and the number of period to run the
% impulse response function for
% Output is the impulse response functions for neutral technology shock

% Parametrization 
beta=params(1,:);  
theta=params(2,:);  
eta=params(3,:);
delta=params(4,:);                     
rhoA=params(5,:);
rhog=params(6,:);
rhoV=params(7,:);
g_y=params(8,:);
sigA=0.0066;            
sigg=params(10,:);
sigg=0.0066;
sigV=params(11,:);
sigV=0.0066;
sigmeas=params(12,:);
nu=params(13,:);
q=params(14,:);
                      
psi = (eta-beta*(1-delta))/eta;

si=theta*(eta-1+delta)/(eta/beta-1+delta);      % investment share of output
sc=(1-si-g_y);                                  % consumption's share of output
k_y=theta*beta/(eta-beta*(1-delta));            % steady state capital to output ratio
    
a1=(beta*nu/eta)/(1-nu/eta);
a2=-((1+beta*(nu^2/eta^2))/(1-nu/eta));
a3=(nu/eta)/(1-nu/eta);
a4=(1-beta*nu/eta);

a5=sc/(1-theta);
a6=eta*k_y/(1-theta);
a7=-((1-delta)*k_y+theta)/(1-theta);
a8=-1/(1-theta);
a9=g_y/(1-theta);
a10=k_y*(1-delta-eta)/(1-theta);

a11=theta*a5;
a12=theta*a6;
a13=theta*a7-theta;
a14=theta*a8-1;
a15=theta*a9;
a16=theta*a10;

b1=a1;
b2=a2-a4*a11;
b3=a3;
b4=a4*a12;
b5=a4*a13;
b6=a4*a14;
b7=a4*a15;
b8=a4*a16;

b9=a11+(1-theta)*psi*a5;
b10=a12+a6*(1-theta)*psi+beta*q*eta;
b11=a13+(1-theta)*psi*a7-(1-theta)*psi-beta*q*eta-a12-eta*q;
b12=a11;
b13=a13-q*eta;
b14=a14-rhoA*(a14+psi+(1-theta)*psi*a8);
b15=a15*(1-rhog)-psi*(1-theta)*rhog*a9;
b16=a16-1-a16*rhoV-psi*(1-theta)*rhoV*a10+(1-delta)*beta*rhoV/eta;


A=[b2 0 b1 0; 0 b11 b9 b10; 1 0 0 0; 0 1 0 0];
B=[-b3 b5 0 b4; 0 b13 b12 0; 0 0 1 0; 0 0 0 1];
C=[b6 b7 b8; b14 b15 b16; 0 0 0; 0 0 0];

invA=inv(A);
D=invA*B;
F=invA*C;

[GinverseBG,H]=eigs(D);        % command eigs sorts the eigenvalues by largest to smallest

H;

% A necessary condition for a unique solution to exist, by Blanchard-Kahn
% (1980) Proposition 1, is that the number of non-predetermined vaiables is
% equal to the number of eigenvalues of D outside the unit circle
% here, this means we need 2 eigenvalues greater than 1

% We're all good here!! Finally!

lambda1H=H(4,4);
lambda2H=H(3,3);
lambda3H=H(2,2);
lambda4H=H(1,1);

Ginv=zeros(4,4);
Ginv(:,1)=GinverseBG(:,4);
Ginv(:,2)=GinverseBG(:,3);
Ginv(:,3)=GinverseBG(:,2);
Ginv(:,4)=GinverseBG(:,1);
G=inv(Ginv);
G11=G(1,1);
G12=G(1,2);
G13=G(1,3);
G14=G(1,4);
G21=G(2,1);
G22=G(2,2);
G23=G(2,3);
G24=G(2,4);
G31=G(3,1);
G32=G(3,2);
G33=G(3,3);
G34=G(3,4);
G41=G(4,1);
G42=G(4,2);
G43=G(4,3);
G44=G(4,4);

J=G*F;
J11=J(1,1);
J12=J(1,2);
J13=J(1,3);
J21=J(2,1);
J22=J(2,2);
J23=J(2,3);
J31=J(3,1);
J32=J(3,2);
J33=J(3,3);
J41=J(4,1);
J42=J(4,2);
J43=J(4,3);

d1=(1-G43*G34/(G44*G33))^(-1)*(G43*G31/(G44*G33)-G41/G44);
d2=(1-G43*G34/(G44*G33))^(-1)*(G43*G32/(G44*G33)-G42/G44);
d3=(1-G43*G34/(G44*G33))^(-1)*(J41/(G44*(rhoA-lambda4H))-G43*J31/(G44*(rhoA-lambda3H)*G33));
d4=(1-G43*G34/(G44*G33))^(-1)*(J42/(G44*(rhog-lambda4H))-G43*J32/(G44*(rhog-lambda3H)*G33));
d5=(1-G43*G34/(G44*G33))^(-1)*(J43/(G44*(rhoV-lambda4H))-G43*J33/(G44*(rhoV-lambda3H)*G33));

d6=-(G31/G33+G34*d1/G33);
d7=-(G32/G33+G34*d2/G33);
d8=J31/((rhoA-lambda3H)*G33)-G34*d3/G33;
d9=J32/((rhog-lambda3H)*G33)-G34*d4/G33;
d10=J33/((rhoV-lambda3H)*G33)-G34*d5/G33;

j1=G11+G13*d6+G14*d1;
j2=G12+G13*d7+G14*d2;
j3=lambda1H*j1;
j4=lambda1H*j2;
j5=(lambda1H-rhoA)*(G13*d8+G14*d3)+J11;
j6=(lambda1H-rhog)*(G13*d9+G14*d4)+J12;
j7=(lambda1H-rhoV)*(G13*d10+G14*d5)+J13;

j8=G21+G23*d6+G24*d1;
j9=G22+G23*d7+G24*d2;
j10=lambda2H*j8;
j11=lambda2H*j9;
j12=(lambda2H-rhoA)*(G23*d8+G24*d3)+J21;
j13=(lambda2H-rhog)*(G23*d9+G24*d4)+J22;
j14=(lambda2H-rhoV)*(G23*d10+G24*d5)+J23;

K1=[j1 j2 0 0 0; j8 j9 0 0 0; 0 0 1 0 0; 0 0 0 1 0; 0 0 0 0 1];
K2=[j3 j4 j5 j6 j7; j10 j11 j12 j13 j14; 0 0 rhoA 0 0; 0 0 0 rhog 0; 0 0 0 0 rhoV];
K3=[0 0 0; 0 0 0; 1 0 0; 0 1 0; 0 0 1];
PI=inv(K1)*K2;
W=inv(K1)*K3;

PI11=PI(1,1);
PI12=PI(1,2);
PI13=PI(1,3);
PI14=PI(1,4);
PI15=PI(1,5);
PI21=PI(2,1);
PI22=PI(2,2);
PI23=PI(2,3);
PI24=PI(2,4);
PI25=PI(2,5);
PI31=PI(3,1);
PI32=PI(3,2);
PI33=PI(3,3);
PI34=PI(3,4);
PI35=PI(3,5);

m1=a5*PI11+a6*PI21;
m2=a5*PI12+a6*PI22+a7;
m3=a5*PI13+a6*PI23+a8;
m4=a5*PI14+a6*PI24+a9;
m5=a5*PI15+a6*PI25+a10;

m6=(1-theta)*m1;
m7=theta+(1-theta)*m2;
m8=(1-theta)*m3+1;
m9=m4*(1-theta);
m10=m5*(1-theta);

m11=eta*PI21/(eta-1+delta);
m12=eta*PI22/(eta-1+delta)-(1-delta)/(eta-1+delta);
m13=eta*PI23/(eta-1+delta);
m14=eta*PI24/(eta-1+delta);
m15=eta*PI25/(eta-1+delta);

U=[m6 m7 m8 m9 m10; m11 m12 m13 m14 m15;m1 m2 m3 m4 m5; PI11 PI12 PI13 PI14 PI15];

% Now need to compute the IRF for hours

% First, define the variables
irfend=40;              %last period of the IRF
s = zeros(5,irfend);             % state vector for period 0 to 20
                            % row entry denotes time period
                            % variable 1 is capital, variable 2 is
                            % technology
sh=zeros(3,irfend);
sh(1,1)=sigA;

time=zeros(irfend,1);
for j=1:irfend
    time(j,1)=j;
end

f=zeros(4,irfend);             % vector of control variables
             % The order of the variables (4):  y i h c
s(:,1)=W*sh(:,1);                           
f(:,1)=U*s(:,1);
                           
for j = 2:irfend
    s(:,j)=PI*s(:,j-1)+W*sh(:,j);
    f(:,j)=U*s(:,j);
end
yneutralirf=f(1,1:irfend);
ineutralirf=f(2,1:irfend);
hneutralirf=f(3,1:irfend);
cneutralirf=f(4,1:irfend);
% Everything looks as it should - we are happy!

IRFSneutral=[yneutralirf' ineutralirf' hneutralirf' cneutralirf'];

% Graph Impulse Response Functions
%figure('name','Figure 2: Neutral Technology in Habit','NumberTitle','off')
%subplot(221),plot(time,yirf*100),title('Output'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(222),plot(time,iirf*100),title('Investment'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(223),plot(time,hirf*100),title('Hours'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(224),plot(time,cirf*100),title('Consumption'),xlabel('quarters'),ylabel('% dev. from steady state')

% Now look at the case for government spending shocks
sh=zeros(3,irfend);
s=zeros(5,irfend);
sh(2,1)=sigg;

s(:,1)=W*sh(:,1);
f(:,1)=U*s(:,1);

for j=2:irfend
    s(:,j)=PI*s(:,j-1)+W*sh(:,j);
    f(:,j)=U*s(:,j);
end
ygovtirf=f(1,1:irfend);
igovtirf=f(2,1:irfend);
hgovtirf=f(3,1:irfend);
cgovtirf=f(4,1:irfend);

IRFSgovt=[ygovtirf' igovtirf' hgovtirf' cgovtirf'];

% Now look at the case for investment-specific technology shocks
sh=zeros(3,irfend);
s=zeros(5,irfend);
sh(3,1)=sigV;

s(:,1)=W*sh(:,1);
f(:,1)=U*s(:,1);

for j=2:irfend
    s(:,j)=PI*s(:,j-1)+W*sh(:,j);
    f(:,j)=U*s(:,j);
end
yinvestmentirf=f(1,1:irfend);
iinvestmentirf=f(2,1:irfend);
hinvestmentirf=f(3,1:irfend);
cinvestmentirf=f(4,1:irfend);

IRFSishock=[yinvestmentirf' iinvestmentirf' hinvestmentirf' cinvestmentirf'];

%disp ' '
%disp ' '
% disp 'investment response';
%[ineutralirf' iinvestmentirf'];
% Everything looks as it should - neutral shocks generate only a small response in hours worked while
% investment-specific technology shocks generate the bulk of the variation
% We are happy!

% Graph Impulse Response Functions
%figure('name','Figure 2: Habit Formation and Capital Adjustment Costs','NumberTitle','off')
%subplot(221),plot(time,yneutralirf*100,time,ygovtirf*100,':',time,yinvestmentirf*100, '--'),title('Output'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(222),plot(time,ineutralirf*100,time,igovtirf*100,':',time,iinvestmentirf*100, '--'),title('Investment'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(223),plot(time,hneutralirf*100,time,hgovtirf*100,':',time,hinvestmentirf*100, '--'),title('Hours'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(224),plot(time,cneutralirf*100,time,cgovtirf*100,':',time,cinvestmentirf*100, '--'),title('Consumption'),xlabel('quarters'),ylabel('% dev. from steady state'),legend('Neutral Shock','Govt Shock','I-Shock','Location','SouthEast')


% Now solve for omegass
sigepsilon=[sigA^2 0 0; 0 sigg^2 0; 0 0 sigV^2];

CC=W*sigepsilon*W';
OMEGAss=dlyap(PI,CC);

SIGyy2=U*OMEGAss*U';

n=4;            % number of variables 
p=4;            % number of lags in the VAR
q=n*p+1;        % total number of variables in the VAR

SIGyx2=zeros(n,q);
for k=1:p
    SIGyx2(:,(k-1)*n+2:k*n+1)=U*PI^k*OMEGAss*U';
end

% WAS TO LAZY TO AUTOMATE THIS PART - NEED TO SET SIGxx
% MANUALLY!
% Here, it is done for a VAR with 4 lags
SIGxx2=zeros(q,q);
SIGxx2(1,1)=1;
for k=1:p
    SIGxx2(2+(k-1)*n:1+k*n,2+(k-1)*n:1+k*n)=SIGyy2;
end
SIGxx2(2:5,6:9)=U*PI*OMEGAss*U';
SIGxx2(2:5,10:13)=U*PI^2*OMEGAss*U';
SIGxx2(2:5,14:17)=U*PI^3*OMEGAss*U';
SIGxx2(6:9,2:5)=U*OMEGAss*PI'*U';
SIGxx2(6:9,10:13)=U*PI*OMEGAss*U';
SIGxx2(6:9,14:17)=U*PI^2*OMEGAss*U';
SIGxx2(10:13,2:5)=U*OMEGAss*(PI^2)'*U';
SIGxx2(10:13,6:9)=U*OMEGAss*PI'*U';
SIGxx2(10:13,14:17)=U*PI*OMEGAss*U';
SIGxx2(14:17,2:5)=U*OMEGAss*(PI^3)'*U';
SIGxx2(14:17,6:9)=U*OMEGAss*(PI^2)'*U';
SIGxx2(14:17,10:13)=U*OMEGAss*PI'*U';

