% Checking RBC model with DYNARE

% Declare the endogenous variables
var y c i k H g A V;

% Variable definitions
% y = output
% c = consumption
% i = investment
% k = capital
% H = hours worked
% g = government spending
% A = neutral technology
% V = investment-specific technology

% Declare the exogenous variables
varexo epsA epsg epsV;

% Declare the parameters

parameters eta delta beta theta rhoA rhog rhoV sigA sigg sigV kappa g_y;
eta=1.00055;
delta=0.021;
beta=0.99;
theta=0.36;
rhoA=0.95;
rhog=0.8;
rhoV=0.95;
g_y=0.2;
kappa=(eta/beta)-1+delta;
sigA=0.0066;
sigg=0.0066;
sigV=0.0066;

model(linear);
y=(1-g_y-theta*(eta-1+delta)/(eta/beta-1+delta))*c+theta*(eta-1+delta)/(eta/beta-1+delta)*i+g_y*g;
y = A + theta*k(-1)+(1-theta)*H;
A=rhoA*A(-1)+epsA;
g=rhog*g(-1)+epsg;
V=rhoV*V(-1)+epsV;
eta*k=(1-delta)*k(-1)+(eta-1+delta)*i+(eta-1+delta)*V;
c+H=y;
0=(eta/beta)*c-(eta/beta)*c(+1)+kappa*y(+1)-kappa*k+(eta/beta-(1-delta)*rhoV)*V;
end;

% Declare the variances of the shock process
shocks;
var epsA; stderr sigA;
var epsg; stderr sigg;
var epsV; stderr sigV;
end;

check;

stoch_simul(linear,irf=40) y i H c A g V;
