function PHIS=RBCdrawsdetrend(X,Y,zetas,lambda);

% Inputs = X (RHS variable), Y(LHS Variable), zetas(DSGE model parameter 
% vector), lambda (hyperparameter)

% Outputs = PHIs (n*p+1)xnxnsim Coefficient matrix

% This program computes draws from the posterior distribution for the model
% RBC

% Order of parameters in RBC priors is beta, theta, eta, delta, rhoA,
% rhog, rhoV,g_y, sigA, sigg, sigV, sigmeas

%% Generating draws from posterior distribution of PHI
[nsim,junk]=size(zetas);
[T,k]=size(X);
[T,n]=size(Y);
PHIS=zeros(k,n,nsim);                  
PHI=zeros(k*n,1);

for c=1:nsim
    c;
    
    % Computing relevant matrices at draw
    [GAMMAXX,GAMMAYX,GAMMAYY,SIGUtilde]=postRBCdetrend(zetas(c,:)',X,Y,lambda);
    PHItilde=inv(lambda*T*GAMMAXX+X'*X)*(lambda*T*GAMMAYX'+X'*Y);
    
    % Every so often with this model, we get crazy eigenvalues in
    % and if this is the case just take the previous draw
    eigSIGUtilde=eig(SIGUtilde);
    if isreal(eigSIGUtilde)&&(min(eigSIGUtilde)>0) 
        % Take the Choleski Decomposition of SIGUtilde to avoid numerical
        % problems
        D=chol((lambda+1)*T*SIGUtilde);
    
        % Draw from the marginal posterior of Sigmau
        SIGu=iwishrnd(D'*D,(1+lambda)*T-k);
        PHItilde=reshape(PHItilde,n*k,1);
        SIGu=double(kron(SIGu,inv(lambda*T*GAMMAXX+X'*X)));
        ISHOCKeigsSIGu=min(eig(SIGu));
        ISHOCKdetSIGu=det(SIGu);
        if ISHOCKeigsSIGu>(1e-10)
            D=chol(SIGu);
            SIGu=D'*D;
            % Now draw from the posterior of PHI
            PHI=PHItilde+norm_rnd(SIGu);
        else
            PHI=PHItilde;
        end
        
    else
        PHI=PHI;
    end
        
    % Save the draw
    PHIS(:,:,c)=reshape(PHI,k,n);
end
    
    