% Checking the model with DYNARE

% Declare the endogenous variables
var y c k H i g A V p mc r w;

% Variable definitions
% y = output
% c = consumption
% i = investment
% k = capital
% H = hours worked
% i = investment
% g = government spending
% A = neutral technology
% V = investment-specific technology
% p = aggregate price level
% mc = marginal cost
% r = real interest rate
% w = real wage

% Declare the exogenous variables
varexo epsA epsg epsV;

% Declare the parameters

parameters eta delta beta theta rhoA rhog rhoV sigA sigg sigV g_y q k_y si sc chi alpha rss a1 a2 a3;
eta=1.00055;
delta=0.021;
beta=0.99;
theta=0.36;
rhoA=0.95;
rhog=0.8;
rhoV=0.95;
g_y=0.2;
sigA=0.0066;
sigg=0.0066;
sigV=0.0066;
q=10;
chi=6;
alpha=0.8;
k_y=theta*beta/(eta-beta*(1-delta));
si=(eta-1+delta)*k_y;
sc=1-g_y-si;
rss=eta/beta-1+delta;
a1=(1-alpha)*(1-alpha*beta)/(alpha*(1+beta-alpha*beta+alpha*beta*eta^(1-chi)));
a2=beta/(1+beta-alpha*beta+alpha*beta*eta^(1-chi));
a3=eta^(1-chi)/(1+beta-alpha*beta+alpha*beta*eta^(1-chi));

model(linear);
-c+c(+1)=(rss/(rss+1-delta))*r(+1)+(1-(1-delta)*rhoV/(rss+1-delta))*V+q*eta*k(-1)-q*eta*(1+eta/(rss+1-delta))*k+(eta^2*q/(rss+1-delta))*k(+1);
beta*c(+1)+beta*p(+1)=p+c;
mc=theta*r+(1-theta)*w-A;
w-r=k(-1)-h;
p=a1*mc+a2*p(-1)+a3*p(+1);
y = sc*c+eta*k_y*k-(1-delta)*k_y*k(-1)+k_y*(eta-1+delta)*V+g_y*g;
y = A +theta*k(-1)+(1-theta)*H;
w=c;
A=rhoA*A(-1)+epsA;
g=rhog*g(-1)+epsg;
V=rhoV*V(-1)+epsV;
eta*k=(1-delta)*k(-1)+(eta-1+delta)*i+(eta-1+delta)*V;
end;

% Declare the variances of the shock process
shocks;
var epsA; stderr sigA;
var epsg; stderr sigg;
var epsV; stderr sigV;
end;

check;

stoch_simul(linear,irf=40) y i H c;
