% -------------------------------------------------------------------------------------------------------------------------
% THIS PROGRAM CONTAINS THE MATLAB CODES FOR COMPUTING THE CROSS-CORRELATIONS FOR THE MODEL STICKY FOR THE PAPER
% "COMPARING MODELS OF MACROECONOMIC FLUCTUATIONS: HOW BIG ARE THE DIFFERENCES?" BY ANDRA GHENT (2008)
% -------------------------------------------------------------------------------------------------------------------------

params=[0.99;0.36;1.00055;0.021;0.95;0.8;0.95;0.2;0.0066;0.0066;0.0066;0.0066;6;0.8;10];


% Parametrization 
beta=params(1,:);  
theta=params(2,:);  
eta=params(3,:);
delta=params(4,:);                     
rhoA=params(5,:);
rhog=params(6,:);
rhoV=params(7,:);
g_y=params(8,:);      
sigg=params(10,:);
sigV=params(11,:);
sigmeas=params(12,:);
xi=params(13,:);
alpha=params(14,:);
q=params(15,:);

% Steady State
r=eta/beta-1+delta;
k_y=theta/r;
i_y=(eta-1+delta)*k_y;
c_y=1-g_y-(eta-1+delta)*k_y;
                        
a1=(1-alpha)*(1-alpha*beta)/(alpha*(1+beta-alpha*beta+alpha*beta*eta^(1-xi)));
a2=beta/(1+beta-alpha*beta+alpha*beta*eta^(1-xi));
a3=eta^(1-xi)/(1+beta-alpha*beta+alpha*beta*eta^(1-xi));

b1=a1*(1+theta*c_y/(1-theta));
b2=-a1*theta*(1+(theta+(1-delta)*k_y)/(1-theta));
b3=a2;
b4=a3;
b5=a1*theta*eta*k_y/(1-theta);
b6=-a1*(1+theta/(1-theta));
b7=a1*theta*g_y/(1-theta);
b8=a1*theta*(eta-1+delta)*k_y/(1-theta);

b9=1-r*c_y/((1+r-delta)*(1-theta));
b10=-(r*eta*k_y/((r+1-delta)*(1-theta))+eta^2*q/(r+1-delta));
b11=1+r/(r+1-delta);
b12=q*eta;
b13=-(r+1-delta)^(-1)*(r+q*eta^2+r*(theta+(1-delta)*k_y/(1-theta)))-q*eta;
b14=-r*rhoA/((r+1-delta)*(1-theta));
b15=r*g_y*rhog/((r+1-delta)*(1-theta));
b16=1-(1-delta)*rhoV/(r+1-delta)+(r/(r+1-delta))*((eta-1+delta)*k_y/(1-theta))*rhoV;

A=[0 0 0 beta beta; 0 0 0 -b4 0; 0 0 b10 0 b9; 1 0 0 0 0 ; 0 1 0 0 0];

B=[0 0 0 1 1; b3 b2 b5 -1 b1; 0 b12 b13 0 b11; 0 0 0 1 0; 0 0 1 0 0];

C=[0 0 0; b6 b7 b8; b14 b15 b16;0 0 0; 0 0 0];

D=inv(A)*B;
F=inv(A)*C;

[GinverseSP,H]=eigs(D);

% A necessary condition for a unique solution to exist, by Blanchard-Kahn
% (1980) Proposition 1, is that the number of non-predetermined vaiables is
% equal to the number of eigenvalues of D outside the unit circle
% here, this means we need 3 eigenvalues greater than 1

% We're good for all the calibrations I consider
Ginv=zeros(5,5);
Ginv(:,1)=GinverseSP(:,5);
Ginv(:,2)=GinverseSP(:,4);
Ginv(:,3)=GinverseSP(:,3);
Ginv(:,4)=GinverseSP(:,2);
Ginv(:,5)=GinverseSP(:,1);

G=inv(Ginv);

G11=G(1,1);
G12=G(1,2);
G13=G(1,3);
G14=G(1,4);
G15=G(1,5);
G21=G(2,1);
G22=G(2,2);
G23=G(2,3);
G24=G(2,4);
G25=G(2,5);
G31=G(3,1);
G32=G(3,2);
G33=G(3,3);
G34=G(3,4);
G35=G(3,5);
G41=G(4,1);
G42=G(4,2);
G43=G(4,3);
G44=G(4,4);
G45=G(4,5);
G51=G(5,1);
G52=G(5,2);
G53=G(5,3);
G54=G(5,4);
G55=G(5,5);

lambda1H=H(5,5);
lambda2H=H(4,4);
lambda3H=H(3,3);
lambda4H=H(2,2);
lambda5H=H(1,1);

J=G*F;

J11=J(1,1);
J12=J(1,2);
J13=J(1,3);
J21=J(2,1);
J22=J(2,2);
J23=J(2,3);
J31=J(3,1);
J32=J(3,2);
J33=J(3,3);
J41=J(4,1);
J42=J(4,2);
J43=J(4,3);
J51=J(5,1);
J52=J(5,2);
J53=J(5,3);

K=G(3:5,3:5);
L=-G(3:5,1:2);
M=[J31/(rhoA-lambda3H) J32/(rhog-lambda3H) J33/(rhoV-lambda3H); J41/(rhoA-lambda4H) J42/(rhog-lambda4H) J43/(rhoV-lambda4H); J51/(rhoA-lambda5H) J52/(rhog-lambda5H) J53/(rhoV-lambda5H)];
N=inv(K)*L;
O=inv(K)*M;

P=zeros(5,5);
P(1,1)=G11+G13*N(1,1)+G14*N(2,1)+G15*N(3,1);
P(1,2)=G12+G13*N(1,2)+G14*N(2,2)+G15*N(3,2);
P(2,1)=G21+G23*N(1,1)+G24*N(2,1)+G25*N(3,1);
P(2,2)=G22+G23*N(1,2)+G24*N(2,2)+G25*N(3,2);
P(3,3)=1;
P(4,4)=1;
P(5,5)=1;

Q=zeros(5,5);
Q(1,1)=lambda1H*P(1,1);
Q(1,2)=lambda1H*P(1,2);
Q(1,3)=J11+(lambda1H-rhoA)*(G13*O(1,1)+G14*O(2,1)+G15*O(3,1));
Q(1,4)=J12+(lambda1H-rhog)*(G13*O(1,2)+G14*O(2,2)+G15*O(3,2));
Q(1,5)=J13+(lambda1H-rhoV)*(G13*O(1,3)+G14*O(2,3)+G15*O(3,3));
Q(2,1)=lambda2H*P(2,1);
Q(2,2)=lambda2H*P(2,2);
Q(2,3)=J21+(lambda2H-rhoA)*(G23*O(1,1)+G24*O(2,1)+G25*O(3,1));
Q(2,4)=J22+(lambda2H-rhog)*(G23*O(1,2)+G24*O(2,2)+G25*O(3,2));
Q(2,5)=J23+(lambda2H-rhoV)*(G23*O(1,3)+G24*O(2,3)+G25*O(3,3));
Q(3,3)=rhoA;
Q(4,4)=rhog;
Q(5,5)=rhoV;

PI=inv(P)*Q;

W=[0 0 0; 0 0 0; 1 0 0; 0 1 0; 0 0 1];

t1=c_y*N(3,1)/(1-theta)+eta*k_y*PI(2,1)/(1-theta);
t2=c_y*N(3,2)/(1-theta)+eta*k_y*PI(2,2)/(1-theta)-(theta+(1-delta)*k_y)/(1-theta);
t3=c_y*O(3,1)/(1-theta)+eta*k_y*PI(2,3)/(1-theta)-1/(1-theta);
t4=c_y*O(3,2)/(1-theta)+eta*k_y*PI(2,4)/(1-theta)+g_y/(1-theta);
t5=c_y*O(3,3)/(1-theta)+eta*k_y*PI(2,5)/(1-theta)+(eta-1+delta)*k_y/(1-theta);

t6=eta*PI(2,1)/(eta-1+delta);
t7=(eta*PI(2,2)-1+delta)/(eta-1+delta);
t8=eta*PI(2,3)/(eta-1+delta);
t9=eta*PI(2,4)/(eta-1+delta);
t10=eta*PI(2,5)/(eta-1+delta)-1;

U=[(1-theta)*t1 (theta+(1-theta)*t2) ((1-theta)*t3+1) (1-theta)*t4 (1-theta)*t5; t6 t7 t8 t9 t10; t1 t2 t3 t4 t5; N(3,1) N(3,2) O(3,1) O(3,2) O(3,3)];


% Now need to compute the IRFs

% First, define the variables
irfend=40;                  % last period for the IRF
s = zeros(5,irfend);             % state vector for period 0 to irfend
                            % row entry denotes time period
                            % variable 1 is capital, variable 2 is
                            % technology
sh=zeros(3,irfend);
sh(1,1)=sigA;
%for j=2:irfend
%    sh(1,j)=sh(1, j-1)*rhoA;
%end
time=zeros(irfend,1);
for j=1:irfend
   time(j,1)=j;
end
f=zeros(4,irfend);             % vector of control variables
             % The order of the variables (4):  y i h c
s(:,1)=W*sh(:,1);                           
f(:,1)=U*s(:,1);
                           
for j = 2:irfend
    s(:,j)=PI*s(:,j-1)+W*sh(:,j);
    f(:,j)=U*s(:,j);
end
yneutralirf=f(1,1:irfend);
ineutralirf=f(2,1:irfend);
hneutralirf=f(3,1:irfend);
cneutralirf=f(4,1:irfend);
% Everything looks as it should - we are happy!
IRFSneutral=[yneutralirf' ineutralirf' hneutralirf' cneutralirf'];

% Now look at the case for government spending shocks
sh=zeros(3,irfend);
s=zeros(5,irfend);
sh(2,1)=sigg;

s(:,1)=W*sh(:,1);
f(:,1)=U*s(:,1);

for j=2:irfend
    s(:,j)=PI*s(:,j-1)+W*sh(:,j);
    f(:,j)=U*s(:,j);
end
ygovtirf=f(1,1:irfend);
igovtirf=f(2,1:irfend);
hgovtirf=f(3,1:irfend);
cgovtirf=f(4,1:irfend);

IRFSgovt=[ygovtirf' igovtirf' hgovtirf' cgovtirf'];


% Now look at the case for investment-specific technology shocks
sh=zeros(3,irfend);
s=zeros(5,irfend);
sh(3,1)=sigV;
%for j=2:irfend
%    sh(3,j)=sh(3,j-1)*rhoV;
%end
s(:,1)=W*sh(:,1);
f(:,1)=U*s(:,1);

for j=2:irfend
    s(:,j)=PI*s(:,j-1)+W*sh(:,j);
    f(:,j)=U*s(:,j);
end
yinvestmentirf=f(1,1:irfend);
iinvestmentirf=f(2,1:irfend);
hinvestmentirf=f(3,1:irfend);
cinvestmentirf=f(4,1:irfend);

IRFSishock=[yinvestmentirf' iinvestmentirf' hinvestmentirf' cinvestmentirf'];

% Graph Impulse Response Functions
%figure('name','Figure 3: Sticky Price Model with an Unaccommodating Monetary Authority','NumberTitle','off')
%subplot(221),plot(time,yneutralirf*100,time,ygovtirf*100,':',time,yinvestmentirf*100, '--'),title('Output'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(222),plot(time,ineutralirf*100,time,igovtirf*100,':',time,iinvestmentirf*100, '--'),title('Investment'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(223),plot(time,hneutralirf*100,time,hgovtirf*100,':',time,hinvestmentirf*100, '--'),title('Hours'),xlabel('quarters'),ylabel('% dev. from steady state')
%subplot(224),plot(time,cneutralirf*100,time,cgovtirf*100,':',time,cinvestmentirf*100, '--'),title('Consumption'),xlabel('quarters'),ylabel('% dev. from steady state'),legend('Neutral Shock','Govt','I-Shock','Location','SouthEast')
   

% Now solve for omegass
sigepsilon=[sigA^2 0 0; 0 sigg^2 0; 0 0 sigV^2];
CC=W*sigepsilon*W';
OMEGAss=dlyap(PI,CC);

sigmeas=0.5*sigA;
VARMEAS=[sigmeas^2 0 0 0; 0 0 0 0; 0 0 sigmeas^2 0; 0 0 0 sigmeas^2];

SIGyy4=U*OMEGAss*U'+VARMEAS;

n=4;            % number of variables 
p=4;            % number of lags in the VAR
q=n*p+1;        % total number of variables in the VAR

SIGyx4=zeros(n,q);
for k=1:p
    SIGyx4(:,(k-1)*n+2:k*n+1)=U*PI^k*OMEGAss*U';
end

% NEED TO SET SIGxx MANUALLY!
% Here, it is done for a VAR with 4 lags
SIGxx4=zeros(q,q);
SIGxx4(1,1)=1;
for k=1:p
    SIGxx4(2+(k-1)*n:1+k*n,2+(k-1)*n:1+k*n)=SIGyy4;
end
SIGxx4(2:5,6:9)=U*PI*OMEGAss*U';
SIGxx4(2:5,10:13)=U*PI^2*OMEGAss*U';
SIGxx4(2:5,14:17)=U*PI^3*OMEGAss*U';
SIGxx4(6:9,2:5)=U*OMEGAss*PI'*U';
SIGxx4(6:9,10:13)=U*PI*OMEGAss*U';
SIGxx4(6:9,14:17)=U*PI^2*OMEGAss*U';
SIGxx4(10:13,2:5)=U*OMEGAss*(PI^2)'*U';
SIGxx4(10:13,6:9)=U*OMEGAss*PI'*U';
SIGxx4(10:13,14:17)=U*PI*OMEGAss*U';
SIGxx4(14:17,2:5)=U*OMEGAss*(PI^3)'*U';
SIGxx4(14:17,6:9)=U*OMEGAss*(PI^2)'*U';
SIGxx4(14:17,10:13)=U*OMEGAss*PI'*U';

SIGyx=SIGyx4;
SIGyy=SIGyy4;

OwnCorrsc=[SIGyx(4,5)/SIGyy(4,4) SIGyx(4,9)/SIGyy(4,4) SIGyx(4,13)/SIGyy(4,4) SIGyx(4,17)/SIGyy(4,4)];

OwnCorrsy=[SIGyx(1,2)/SIGyy(1,1) SIGyx(1,6)/SIGyy(1,1) SIGyx(1,10)/SIGyy(1,1) SIGyx(1,14)/SIGyy(1,1)];
Corrsyi=[SIGyx(1,3)/(sqrt(SIGyy(1,1)*SIGyy(2,2))) SIGyx(1,7)/(sqrt(SIGyy(1,1)*SIGyy(2,2))) SIGyx(1,11)/(sqrt(SIGyy(1,1)*SIGyy(2,2))) SIGyx(1,15)/(sqrt(SIGyy(1,1)*SIGyy(2,2)))];
Corrsyh=[SIGyx(1,4)/(sqrt(SIGyy(1,1)*SIGyy(3,3))) SIGyx(1,8)/(sqrt(SIGyy(1,1)*SIGyy(3,3))) SIGyx(1,12)/(sqrt(SIGyy(1,1)*SIGyy(3,3))) SIGyx(1,16)/(sqrt(SIGyy(1,1)*SIGyy(3,3)))];
Corrsyc=[SIGyx(1,5)/(sqrt(SIGyy(1,1)*SIGyy(4,4))) SIGyx(1,9)/(sqrt(SIGyy(1,1)*SIGyy(4,4))) SIGyx(1,13)/(sqrt(SIGyy(1,1)*SIGyy(4,4))) SIGyx(1,17)/(sqrt(SIGyy(1,1)*SIGyy(4,4)))];

Corrsiy=[SIGyx(2,2)/(sqrt(SIGyy(1,1)*SIGyy(2,2))) SIGyx(2,6)/(sqrt(SIGyy(1,1)*SIGyy(2,2))) SIGyx(2,10)/(sqrt(SIGyy(1,1)*SIGyy(2,2))) SIGyx(2,14)/(sqrt(SIGyy(1,1)*SIGyy(2,2)))];
OwnCorrsi=[SIGyx(2,3)/SIGyy(2,2) SIGyx(2,7)/SIGyy(2,2) SIGyx(2,11)/SIGyy(2,2) SIGyx(2,15)/SIGyy(2,2)];
Corrsih=[SIGyx(2,4)/(sqrt(SIGyy(2,2)*SIGyy(3,3))) SIGyx(2,8)/(sqrt(SIGyy(2,2)*SIGyy(3,3))) SIGyx(2,12)/(sqrt(SIGyy(2,2)*SIGyy(3,3))) SIGyx(2,16)/(sqrt(SIGyy(2,2)*SIGyy(3,3)))];
Corrsic=[SIGyx(2,5)/(sqrt(SIGyy(2,2)*SIGyy(4,4))) SIGyx(2,9)/(sqrt(SIGyy(2,2)*SIGyy(4,4))) SIGyx(2,13)/(sqrt(SIGyy(2,2)*SIGyy(4,4))) SIGyx(2,17)/(sqrt(SIGyy(2,2)*SIGyy(4,4)))];

Corrshy=[SIGyx(3,2)/(sqrt(SIGyy(1,1)*SIGyy(3,3))) SIGyx(3,6)/(sqrt(SIGyy(1,1)*SIGyy(3,3))) SIGyx(3,10)/(sqrt(SIGyy(1,1)*SIGyy(3,3))) SIGyx(3,14)/(sqrt(SIGyy(1,1)*SIGyy(3,3)))];
Corrshi=[SIGyx(3,3)/(sqrt(SIGyy(3,3)*SIGyy(2,2))) SIGyx(3,7)/(sqrt(SIGyy(3,3)*SIGyy(2,2))) SIGyx(3,11)/(sqrt(SIGyy(3,3)*SIGyy(2,2))) SIGyx(3,15)/(sqrt(SIGyy(3,3)*SIGyy(2,2)))];
OwnCorrsh=[SIGyx(3,4)/SIGyy(3,3) SIGyx(3,8)/SIGyy(3,3) SIGyx(3,12)/SIGyy(3,3) SIGyx(3,16)/SIGyy(3,3)];
Corrshc=[SIGyx(3,5)/(sqrt(SIGyy(3,3)*SIGyy(4,4))) SIGyx(3,9)/(sqrt(SIGyy(3,3)*SIGyy(4,4))) SIGyx(3,13)/(sqrt(SIGyy(3,3)*SIGyy(4,4))) SIGyx(3,17)/(sqrt(SIGyy(3,3)*SIGyy(4,4)))];

Corrscy=[SIGyx(4,2)/(sqrt(SIGyy(1,1)*SIGyy(4,4))) SIGyx(4,6)/(sqrt(SIGyy(1,1)*SIGyy(4,4))) SIGyx(4,10)/(sqrt(SIGyy(1,1)*SIGyy(4,4))) SIGyx(4,14)/(sqrt(SIGyy(1,1)*SIGyy(4,4)))];
Corrsci=[SIGyx(4,3)/(sqrt(SIGyy(4,4)*SIGyy(2,2))) SIGyx(4,7)/(sqrt(SIGyy(4,4)*SIGyy(2,2))) SIGyx(4,11)/(sqrt(SIGyy(4,4)*SIGyy(2,2))) SIGyx(4,15)/(sqrt(SIGyy(4,4)*SIGyy(2,2)))];
Corrsch=[SIGyx(4,4)/(sqrt(SIGyy(4,4)*SIGyy(3,3))) SIGyx(4,8)/(sqrt(SIGyy(4,4)*SIGyy(3,3))) SIGyx(4,12)/(sqrt(SIGyy(4,4)*SIGyy(3,3))) SIGyx(4,16)/(sqrt(SIGyy(4,4)*SIGyy(3,3)))];
OwnCorrsc=[SIGyx(4,5)/SIGyy(4,4) SIGyx(4,9)/SIGyy(4,4) SIGyx(4,13)/SIGyy(4,4) SIGyx(4,17)/SIGyy(4,4)];

Ycorrs=[OwnCorrsy';Corrsyi';Corrsyh';Corrsyc'];
Icorrs=[Corrsiy';OwnCorrsi';Corrsih';Corrsic'];
Hcorrs=[Corrshy';Corrshi';OwnCorrsh';Corrshc'];
Ccorrs=[Corrscy';Corrsci';Corrsch';OwnCorrsc'];
CorrsSTICKY=[Ycorrs Icorrs Hcorrs Ccorrs];
csvwrite('STICKYCorrs.csv',CorrsSTICKY);
