function PHIS=STICKYdraws(X,Y,zetas,lambda);

% Inputs = X (RHS variable), Y(LHS Variable), zetas (draws from DSGE
% priors), lambda (hyperparameter)

% Outputs = PHIs (n*p+1)xn Coefficient matrix

% This program computes draws from the posterior distribution for the model
% RBC

    % Order of parameters in RBC priors is beta, theta, eta, delta, rhoA,
    % rhog, rhoV,g_y, sigA, sigg, sigV, sigmeas

%% Generating draws from posterior distribution of PHI
[nsim,junk]=size(zetas);
[T,k]=size(X);
[T,n]=size(Y);
PHIS=zeros(k,n,nsim);                  
PHI=zeros(k*n,1);


% Initial draw
for ii=1:nsim
    ii;
    % Computing relevant matrices at draw
    [GAMMAXX,GAMMAYX,GAMMAYY,SIGUtilde]=postSTICKY(zetas(ii,:)',X,Y,lambda);
    XX=X'*X;
    GAMMAXXinv=inv(T*GAMMAXX);
    XXinv=inv(XX);
    PHItilde=inv(lambda*T*GAMMAXX+XX)*(lambda*T*GAMMAYX'+X'*Y);
    eig(SIGUtilde);
    det(SIGUtilde);
    if isreal(eig(SIGUtilde))&&(min(eig(SIGUtilde))>4e-006)
        D=chol((lambda+1)*T*SIGUtilde);
        % Now draw from the posterior of PHI
        % Draw from the marginal posterior of Sigmau
        %SIGu=iwishrnd((lambda+1)*T*newsigu,(1+lambda)*T-k);
        SIGu=iwishrnd(D'*D,(1+lambda)*T-k);
        testing=norm_rnd(SIGu);
        % Check to make sure that SIGu is positive definite
        eigsSIGu=eig(SIGu);
        eigsSIGutilde=eig(SIGUtilde);
        detSIGu=det(SIGu);
        % if SIGu is no positive definite, simply use SIGUtilde
        if min(eigsSIGu)<0
            SIGu=SIGUtilde;
        end
        PHItilde=reshape(PHItilde,n*k,1);
        SIGu=double(kron(SIGu,inv(lambda*T*GAMMAXX+X'*X)));
        % Check to make sure that SIGu is positive definite
        eigsSIGubig=min(eig(SIGu));
        detSIGubig=det(SIGu);
        % if SIGu is not positive definite, or there are computation problems 
        % with the matrix (too close to being singular) simply use PHItilde 
        % instead of actually drawing from posterior distribution of PHI
        if (min(eigsSIGubig)<0)||(detSIGubig<1e-185)
            SIGu=kron(SIGUtilde,inv(lambda*T*GAMMAXX+X'*X));
            PHI=PHItilde;
        else
            D=chol(SIGu);
            SIGu=D'*D;
            PHI=PHItilde+norm_rnd(SIGu);
        end
    else
        PHI=PHI;
    end
    %[test5,test6]=size(SIGu)
    PHIS(:,:,ii)=reshape(PHI,k,n);
end
    
    