% This is the master file for doing the forecasts in the paper "Comparing
% Models of Economic Fluctuations: How Big are the Differences?" (2008) by
% Andra Ghent

%% Preliminaries
close all;
clear all;
clc;

% load the data
load FinalData.txt;

p=4;                    % Number of lags

% WARNING: CHANGING n or p REQUIRES CHANGING MANY MORE PROGRAMS BESIDES THE
% MASTER FILE
n=4;                    % Number of variables in the VAR 

k=n*p+1;

T=length(FinalData);    % Total number of observations

y=log(FinalData(:,:));

Test=160;   % Number of Periods used to estimate the model
maxhor=20;  % Maximum number of Periods to forecast ahead

Nsim=5;    % Number of draws from posterior distribution of VAR parameters

%% Hyperparameters
lambdas=[0.25;0.5;1;5;10;20];           % Hyperparameter for DSGE Priors
mew6=5;                                 % Hyperparameter for Cointegration prior
theta=0.2;                              % Hyperparameter 1 for Litterman prior
lambdaLit=0.05;                         % Hyperparameter 2 for Litterman prior

nlambdas=length(lambdas);

%% END OF MANUAL PART

% Preallocating memory for certain variables to maximize speed

yforcRBC=zeros(T-Test,n,maxhor,nlambdas);   % First index denotes date, second index
% denotes which variable, Third index denotes how many periods ahead,
% Fourth index denotes which hyperparameter
yforcHABIT=zeros(T-Test,n,maxhor,nlambdas);
yforcISHOCK=zeros(T-Test,n,maxhor,nlambdas);
yforcSTICKY=zeros(T-Test,n,maxhor,nlambdas);
yforcSTICKYACC=zeros(T-Test,n,maxhor,nlambdas);
yforcCOMBO=zeros(T-Test,n,maxhor,nlambdas);
yforcRBCINF=zeros(T-Test,n,maxhor); 
yforcHABITINF=zeros(T-Test,n,maxhor);
yforcISHOCKINF=zeros(T-Test,n,maxhor);
yforcSTICKYINF=zeros(T-Test,n,maxhor);
yforcSTICKYACCINF=zeros(T-Test,n,maxhor);
yforcUNRES=zeros(T-Test,n,maxhor);
yforcCOINT=zeros(T-Test,n,maxhor);
yforcLIT=zeros(T-Test,n,maxhor);

% Store the forecast errors
eforcRBC=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
eforcHABIT=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
eforcISHOCK=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
eforcSTICKY=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
eforcCOMBO=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
eforcSTICKYACC=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
eforcRBCINF=zeros(T-Test-maxhor-p,n,maxhor);
eforcHABITINF=zeros(T-Test-maxhor-p,n,maxhor);
eforcISHOCKINF=zeros(T-Test-maxhor-p,n,maxhor);
eforcSTICKYINF=zeros(T-Test-maxhor-p,n,maxhor);
eforcSTICKYACCINF=zeros(T-Test-maxhor-p,n,maxhor);
eforcUNRES=zeros(T-Test-maxhor-p,n,maxhor);
eforcCOINT=zeros(T-Test-maxhor-p,n,maxhor);
eforcLIT=zeros(T-Test-maxhor-p,n,maxhor);

% Store the squared forecast errors
sesRBC=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
sesHABIT=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
sesISHOCK=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
sesSTICKY=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
sesCOMBO=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
sesSTICKYACC=zeros(T-Test-maxhor-p,n,maxhor,nlambdas);
sesRBCINF=zeros(T-Test-maxhor-p,n,maxhor);
sesHABITINF=zeros(T-Test-maxhor-p,n,maxhor);
sesISHOCKINF=zeros(T-Test-maxhor-p,n,maxhor);
sesSTICKYINF=zeros(T-Test-maxhor-p,n,maxhor);
sesSTICKYACCINF=zeros(T-Test-maxhor-p,n,maxhor);
sesUNRES=zeros(T-Test-maxhor-p,n,maxhor);
sesCOINT=zeros(T-Test-maxhor-p,n,maxhor);
sesLIT=zeros(T-Test-maxhor-p,n,maxhor);

PHISUNRES=zeros(k,n);
Xdraw=zeros(Test,k);
Ydraw=zeros(Test,n);

%% Draws from DSGE model parameter priors
sigs=zeros(Nsim,4);
sigsISHOCK=zeros(Nsim,4);

tic
% Common parameters
thetas=betarnd(18,32,Nsim,1);
etas=1.00055+(0.0001)*randn(Nsim,1);
deltas=betarnd(12.6,587.4,Nsim,1);
rhoAs=betarnd(95,5,Nsim,1);
rhogs=betarnd(32,8,Nsim,1);
rhoVs=betarnd(47.5,2.5,Nsim,1);
g_ys=betarnd(20,80,Nsim,1);
    
% Variance parameters for RBC, HABIT, STICKY, and STICKYACC
% Using Stephane Adjemian's code to get the parameters for the
% inverse gamma distribution for given mean and standard deviation
[s,nu]=inverse_gamma_specification(0.0066,0.005,2);
for ii=1:Nsim
    sigs(ii,1)=iwishrnd(s,nu);
    sigs(ii,2)=iwishrnd(s,nu);
    sigs(ii,3)=iwishrnd(s,nu);
    sigs(ii,4)=iwishrnd(s,nu);
end

% Variance parameters for ISHOCK
[s,nu]=inverse_gamma_specification(0.0044,0.005,2);
[s1,nu1]=inverse_gamma_specification(0.0066*2,0.005,2);
for ii=1:Nsim
    sigsISHOCK(ii,1)=iwishrnd(s,nu);
    sigsISHOCK(ii,2)=iwishrnd(s,nu);
    sigsISHOCK(ii,3)=iwishrnd(s1,nu1);
    sigsISHOCK(ii,4)=iwishrnd(s,nu);
end

% Habit and K adj. costs parameters for HABIT
nus=betarnd(14,6,Nsim,1);
qsHABIT=max(0,25+5*randn(Nsim,1));

% Psi, alpha and K adj. costs parameters for STICKY and STICKYACC
psis=max(0,6+2*randn(Nsim,1));
% Exclude 1 for stickiness parameter so take min of 0.999 and betarnd
alphas=min(0.999,betarnd(2,0.5,Nsim,1));
qsSTICKY=max(0,10+5*randn(Nsim,1));
    
% RhoM parameters for STICKYACC
rhoMs=betarnd(50,50,Nsim,1);
    
% Parameter matrices for each model
RBCzetas=[thetas etas deltas rhoAs rhogs rhoVs g_ys sigs];
HABITzetas=[thetas etas deltas rhoAs rhogs rhoVs g_ys sigs nus qsHABIT];
ISHOCKzetas=[thetas etas deltas rhoAs rhogs rhoVs g_ys sigsISHOCK];
STICKYzetas=[thetas etas deltas rhoAs rhogs rhoVs g_ys sigs psis alphas qsSTICKY];
STICKYACCzetas=[thetas etas deltas rhoAs rhogs rhoVs g_ys sigs psis alphas qsSTICKY rhoMs];

%% Loop over every forecasting period
for i=1:(T-Test-maxhor-p)
    i;
    % ---------------------------------------------------------------------
    % ESTIMATION
    % ---------------------------------------------------------------------
    % Raw data
    hours=y(1:Test+i+p-1,3);
    output=y(1:Test+i+p-1,1);
    investment=(y(1:Test+i+p-1,2));
    consumption=(y(1:Test+i+p-1,4));
    % Matrix for detrending
    X=[ones(Test+i+p-1,1) (1:(Test+i+p-1))'];
    % Detrending Coeffs
    trendy=(X'*X)^(-1)*X'*output;
    trendi=(X'*X)^(-1)*X'*investment;
    trendh=(X'*X)^(-1)*X'*hours;
    trendc=(X'*X)^(-1)*X'*consumption;
    trends=[trendy trendi trendh trendc];
    output=output-X*trendy;
    investment=investment-X*trendi;
    hours=hours-X*trendh;
    consumption=consumption-X*trendc;
    ynew=[output investment hours consumption];
    Xdraw=[ones(Test,1) ynew(i+p-1:i+p-2+Test,:) ynew(i+p-2:i+p-3+Test,:) ynew(i+p-3:i+p-4+Test,:) ynew(i+p-4:i+p-5+Test,:)];
    Ydraw=ynew(i+p:i+p-1+Test,:);
    
    % Unrestricted VAR coefficients
    PHISUNRES=(Xdraw'*Xdraw)^(-1)*Xdraw'*Ydraw;
    
    % Dummy observations to implement the cointegration prior
    Xdumcoint=[1 mean(Ydraw) mean(Ydraw) mean(Ydraw) mean(Ydraw)];
    SIGxxCoint=Xdumcoint'*Xdumcoint;
    Ydumcoint=mean(Ydraw);
    SIGyxCoint=Xdumcoint'*Ydumcoint;
    PHISCOINT=(mew6*(Test-p)*SIGxxCoint+Xdraw'*Xdraw)^(-1)*(mew6*(Test-p)*SIGyxCoint+Xdraw'*Ydraw);
    
    % Litterman Prior
    % Set up is to do it equation by equation rather than in matrix form
    % Calculate sigma to be used in the prior's covariance matrix (use
    % univariate AR(p) as in Litterman (1986) and setting mean value of the
    % prior
    sig=zeros(n,1);
    rho=zeros(n,p+1);
    Xsig=zeros(Test,p+1);
    Ysig=zeros(Test,1);
    e=zeros(Test,1);
    gammapri=zeros(k,n); 
    for m=1:n
        Xsig(:,1)=ones(Test,1);
        for r=1:p
            Xsig(:,1+r)=ynew(i+p-r:i+p-r-1+Test,m);
        end
        Ysig(:,1)=ynew(i+p:i+p-1+Test,m);
        rho(m,:)=(Xsig(:,:)'*Xsig(:,:))^(-1)*Xsig(:,:)'*Ysig(:,:);
        e(:,:)=Ysig(:,:)-Xsig(:,:)*rho(m,:)';
        sig(m,:)=e(:,:)'*e(:,:)/(Test-p-1);
        gammapri(m+1,m)=1;
    end
    % Variance parameters for Litterman prior
    delta=zeros(n*p,n);
    for m=1:n
        for w=1:p
            for s=1:n
                delta(n*(w-1)+s,m)=theta*lambdaLit/(w*sig(s,:)^0.5);       % cross lags
            end
        end
        delta(:,m)=delta(:,m)*sig(m)^0.5;
        for w=1:p      
            delta(n*(w-1)+m,m)=lambdaLit/w;                                  % own lags
        end
    end
    R=zeros(k,k,n);
    for z=1:n
        R(1,1,z)=lambdaLit;
        for m=2:k
            R(m,m,z)=lambdaLit/delta(m-1,z);
        end
    end
    Kvec=zeros(n,1);
    for z=1:n
        Kvec(z,1)=sig(z,:)/lambdaLit^2;
    end
    % stack the gammas
    gammaprior=reshape(gammapri,n*k,1); 
    gammalit=zeros(k*n,1);
    for z=1:n
        Yz=ynew(p+i:i+p-1+Test,z);
        gammalit((z-1)*k+1:z*k,1)=(Xdraw'*Xdraw+Kvec(z,1)*R(:,:,z)'*R(:,:,z))^(-1)*(R(:,:,z)'*gammaprior((z-1)*k+1:z*k,:)*Kvec(z,1)+Xdraw'*Yz);
    end
    PHISLIT=reshape(gammalit,k,n);
    clear gammaprior
    
    % Generating VAR coefficients for each draw from the priors
    for xx=1:length(lambdas);
        lambda=lambdas(xx,1);
        PHISRBC=RBCdrawsdetrend(Xdraw,Ydraw,RBCzetas,lambda);       % PHIS is a three dimensional array with third dimension being each draw
        PHISHABIT=HABITdrawsdetrend(Xdraw,Ydraw,HABITzetas,lambda);
        PHISISHOCK=RBCdrawsdetrend(Xdraw,Ydraw,ISHOCKzetas,lambda);
        PHISSTICKY=STICKYdrawsdetrend(Xdraw,Ydraw,STICKYzetas,lambda);
        PHISSTICKYACC=STICKYACCdrawsdetrend(Xdraw,Ydraw,STICKYACCzetas,lambda);
        PHISRBCINF=RBCdrawsdetrend_INF(RBCzetas);
        PHISHABITINF=HABITdrawsdetrend_INF(HABITzetas);
        PHISISHOCKINF=RBCdrawsdetrend_INF(ISHOCKzetas);
        PHISSTICKYINF=STICKYdrawsdetrend_INF(STICKYzetas);
        PHISSTICKYACCINF=STICKYACCdrawsdetrend_INF(STICKYACCzetas);
    
    
    % ---------------------------------------------------------------------
    % ONE PERIOD AHEAD FORECASTS
    % ---------------------------------------------------------------------
    
        xRBC=Xdraw(length(Xdraw):length(Xdraw),2:(n*p+1)); % x is a n*p+1 by 1 row vector
        xUNRES=xRBC;
        xHABIT=xRBC;
        xISHOCK=xRBC;
        xSTICKY=xRBC;
        xSTICKYACC=xRBC;
        xRBCINF=xRBC;
        xHABITINF=xRBC;
        xISHOCKINF=xRBC;
        xSTICKYINF=xRBC;
        xSTICKYACCINF=xRBC;
        xCOINT=xRBC;
        xLIT=xRBC;
        yhatsRBC=zeros(Nsim,n);
        yhatsHABIT=zeros(Nsim,n);
        yhatsISHOCK=zeros(Nsim,n);
        yhatsSTICKY=zeros(Nsim,n);
        yhatsSTICKYACC=zeros(Nsim,n);
        yhatsRBCINF=zeros(Nsim,n);
        yhatsHABITINF=zeros(Nsim,n);
        yhatsISHOCKINF=zeros(Nsim,n);
        yhatsSTICKYINF=zeros(Nsim,n);
        yhatsSTICKYACCINF=zeros(Nsim,n);
        for b=1:Nsim
            yhatsRBC(b,:)=[1 xRBC]*PHISRBC(:,:,b);
            yhatsHABIT(b,:)=[1 xHABIT]*PHISHABIT(:,:,b);
            yhatsISHOCK(b,:)=[1 xISHOCK]*PHISISHOCK(:,:,b);
            yhatsSTICKY(b,:)=[1 xSTICKY]*PHISSTICKY(:,:,b);
            yhatsSTICKYACC(b,:)=[1 xSTICKYACC]*PHISSTICKYACC(:,:,b);
            yhatsRBCINF(b,:)=[1 xRBC]*PHISRBCINF(:,:,b);
            yhatsHABITINF(b,:)=[1 xHABIT]*PHISHABITINF(:,:,b);
            yhatsISHOCKINF(b,:)=[1 xISHOCK]*PHISISHOCKINF(:,:,b);
            yhatsSTICKYINF(b,:)=[1 xSTICKY]*PHISSTICKYINF(:,:,b);
            yhatsSTICKYACCINF(b,:)=[1 xSTICKYACC]*PHISSTICKYACCINF(:,:,b);
        end
        yforcRBC(i,:,1,xx)=mean(yhatsRBC,1);
        yforcHABIT(i,:,1,xx)=mean(yhatsHABIT,1);
        yforcISHOCK(i,:,1,xx)=mean(yhatsISHOCK,1);
        yforcSTICKY(i,:,1,xx)=mean(yhatsSTICKY,1);
        yforcSTICKYACC(i,:,1,xx)=mean(yhatsSTICKYACC,1);
        yforcRBCINF(i,:,1)=mean(yhatsRBCINF,1);
        yforcHABITINF(i,:,1)=mean(yhatsHABITINF,1);
        yforcISHOCKINF(i,:,1)=mean(yhatsISHOCKINF,1);
        yforcSTICKYINF(i,:,1)=mean(yhatsSTICKYINF,1);
        yforcCOMBO(i,:,1,xx)=(yforcRBC(i,:,1,xx)+yforcHABIT(i,:,1,xx)+yforcISHOCK(i,:,1,xx)+yforcSTICKY(i,:,1,xx))/4;
        yforcSTICKYACCINF(i,:,1)=mean(yhatsSTICKYACCINF,1);
        eforcRBC(i,:,1,xx)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcRBC(i,:,1,xx);
        eforcHABIT(i,:,1,xx)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcHABIT(i,:,1,xx);
        eforcISHOCK(i,:,1,xx)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcISHOCK(i,:,1,xx);
        eforcSTICKY(i,:,1,xx)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcSTICKY(i,:,1,xx);
        eforcCOMBO(i,:,1,xx)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcCOMBO(i,:,1,xx);
        eforcSTICKYACC(i,:,1,xx)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcSTICKYACC(i,:,1,xx);
        eforcRBCINF(i,:,1)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcRBCINF(i,:,1);
        eforcHABITINF(i,:,1)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcHABITINF(i,:,1);
        eforcISHOCKINF(i,:,1)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcISHOCKINF(i,:,1);
        eforcSTICKYINF(i,:,1)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcSTICKYINF(i,:,1);
        eforcSTICKYACCINF(i,:,1)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcSTICKYACCINF(i,:,1);
        sesRBC(i,:,1,xx)=eforcRBC(i,:,1,xx).^2;
        sesHABIT(i,:,1,xx)=eforcHABIT(i,:,1,xx).^2;
        sesISHOCK(i,:,1,xx)=eforcISHOCK(i,:,1,xx).^2;
        sesSTICKY(i,:,1,xx)=eforcSTICKY(i,:,1,xx).^2;
        sesCOMBO(i,:,1,xx)=eforcCOMBO(i,:,1,xx).^2;
        sesSTICKYACC(i,:,1,xx)=eforcSTICKYACC(i,:,1,xx).^2;
        sesRBCINF(i,:,1)=eforcRBCINF(i,:,1).^2;
        sesHABITINF(i,:,1)=eforcHABITINF(i,:,1).^2;
        sesISHOCKINF(i,:,1)=eforcISHOCKINF(i,:,1).^2;
        sesSTICKYINF(i,:,1)=eforcSTICKYINF(i,:,1).^2;
        sesSTICKYACCINF(i,:,1)=eforcSTICKYACCINF(i,:,1).^2;
        yforcUNRES(i,:,1)=[1 xUNRES]*PHISUNRES;
        yforcCOINT(i,:,1)=[1 xCOINT]*PHISCOINT;
        yforcLIT(i,:,1)=[1 xLIT]*PHISLIT;
        eforcUNRES(i,:,1)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcUNRES(i,:,1);
        eforcCOINT(i,:,1)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcCOINT(i,:,1);
        eforcLIT(i,:,1)=y(i+p+Test:i+p+Test,:)-[1;(i+p+Test)]'*trends-yforcLIT(i,:,1);
        sesUNRES(i,:,1)=eforcUNRES(i,:,1).^2;
        sesCOINT(i,:,1)=eforcCOINT(i,:,1).^2;
        sesLIT(i,:,1)=eforcLIT(i,:,1).^2;
    
        % ---------------------------------------------------------------------
        % j PERIOD AHEAD FORECASTS
        % ---------------------------------------------------------------------
        for j=2:maxhor
            xRBC=[yforcRBC(i,:,j-1,xx) xRBC(:,1:n*(p-1))];
            xHABIT=[yforcHABIT(i,:,j-1,xx) xHABIT(:,1:n*(p-1))];
            xISHOCK=[yforcISHOCK(i,:,j-1,xx) xISHOCK(:,1:n*(p-1))];
            xSTICKY=[yforcSTICKY(i,:,j-1,xx) xSTICKY(:,1:n*(p-1))];
            xSTICKYACC=[yforcSTICKYACC(i,:,j-1,xx) xSTICKYACC(:,1:n*(p-1))];
            xRBCINF=[yforcRBCINF(i,:,j-1) xRBCINF(:,1:n*(p-1))];
            xHABITINF=[yforcHABITINF(i,:,j-1) xHABITINF(:,1:n*(p-1))];
            xISHOCKINF=[yforcISHOCKINF(i,:,j-1) xISHOCKINF(:,1:n*(p-1))];
            xSTICKYINF=[yforcSTICKYINF(i,:,j-1) xSTICKYINF(:,1:n*(p-1))];
            xSTICKYACCINF=[yforcSTICKYACCINF(i,:,j-1) xSTICKYACCINF(:,1:n*(p-1))];
            xUNRES=[yforcUNRES(i,:,j-1) xUNRES(:,1:n*(p-1))];
            xCOINT=[yforcCOINT(i,:,j-1) xCOINT(:,1:n*(p-1))];
            xLIT=[yforcLIT(i,:,j-1) xLIT(:,1:n*(p-1))];
            for c=1:Nsim
                yhatsRBC(c,:)=[1 xRBC]*PHISRBC(:,:,c);
                yhatsHABIT(c,:)=[1 xHABIT]*PHISHABIT(:,:,c);
                yhatsISHOCK(c,:)=[1 xISHOCK]*PHISISHOCK(:,:,c);
                yhatsSTICKY(c,:)=[1 xSTICKY]*PHISSTICKY(:,:,c);
                yhatsSTICKYACC(c,:)=[1 xSTICKYACC]*PHISSTICKYACC(:,:,c);
                yhatsRBCINF(c,:)=[1 xRBCINF]*PHISRBCINF(:,:,c);
                yhatsHABITINF(c,:)=[1 xHABITINF]*PHISHABITINF(:,:,c);
                yhatsISHOCKINF(c,:)=[1 xISHOCKINF]*PHISISHOCKINF(:,:,c);
                yhatsSTICKYINF(c,:)=[1 xSTICKYINF]*PHISSTICKYINF(:,:,c);
                yhatsSTICKYACCINF(c,:)=[1 xSTICKYACCINF]*PHISSTICKYACCINF(:,:,c);
            end
            yforcRBC(i,:,j,xx)=mean(yhatsRBC,1);
            yforcHABIT(i,:,j,xx)=mean(yhatsHABIT,1);
            yforcISHOCK(i,:,j,xx)=mean(yhatsISHOCK,1);
            yforcSTICKY(i,:,j,xx)=mean(yhatsSTICKY,1);
            yforcCOMBO(i,:,j,xx)=(yforcRBC(i,:,j,xx)+yforcHABIT(i,:,j,xx)+yforcISHOCK(i,:,j,xx)+yforcSTICKY(i,:,j,xx))/4;
            yforcSTICKYACC(i,:,j,xx)=mean(yhatsSTICKYACC,1);
            yforcRBCINF(i,:,j)=mean(yhatsRBCINF,1);
            yforcHABITINF(i,:,j)=mean(yhatsHABITINF,1);
            yforcISHOCKINF(i,:,j)=mean(yhatsISHOCKINF,1);
            yforcSTICKYINF(i,:,j)=mean(yhatsSTICKYINF,1);
            yforcSTICKYACCINF(i,:,j)=mean(yhatsSTICKYACCINF,1);
            yforcUNRES(i,:,j)=[1 xUNRES]*PHISUNRES;
            yforcCOINT(i,:,j)=[1 xCOINT]*PHISCOINT;
            yforcLIT(i,:,j)=[1 xLIT]*PHISLIT;
            eforcRBC(i,:,j,xx)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcRBC(i,:,j,xx);
            eforcHABIT(i,:,j,xx)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcHABIT(i,:,j,xx);
            eforcISHOCK(i,:,j,xx)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcISHOCK(i,:,j,xx);
            eforcSTICKY(i,:,j,xx)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcSTICKY(i,:,j,xx);
            eforcCOMBO(i,:,j,xx)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcCOMBO(i,:,j,xx);
            eforcSTICKYACC(i,:,j,xx)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcSTICKYACC(i,:,j,xx);
            eforcRBCINF(i,:,j)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcRBCINF(i,:,j);
            eforcHABITINF(i,:,j)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcHABITINF(i,:,j);
            eforcISHOCKINF(i,:,j)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcISHOCKINF(i,:,j);
            eforcSTICKYINF(i,:,j)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcSTICKYINF(i,:,j);
            eforcSTICKYACCINF(i,:,j)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcSTICKYACCINF(i,:,j);
            eforcUNRES(i,:,j)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcUNRES(i,:,j);
            eforcCOINT(i,:,j)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcCOINT(i,:,j);
            eforcLIT(i,:,j)=y(i+p+Test+j-1:i+p+Test+j-1,:)-[1;(i+p+Test)]'*trends-yforcLIT(i,:,j);
            sesRBC(i,:,j,xx)=eforcRBC(i,:,j,xx).^2;
            sesHABIT(i,:,j,xx)=eforcHABIT(i,:,j,xx).^2;
            sesISHOCK(i,:,j,xx)=eforcISHOCK(i,:,j,xx).^2;
            sesSTICKY(i,:,j,xx)=eforcSTICKY(i,:,j,xx).^2;
            sesCOMBO(i,:,j,xx)=eforcCOMBO(i,:,j,xx).^2;
            sesSTICKYACC(i,:,j,xx)=eforcSTICKYACC(i,:,j,xx).^2;
            sesRBCINF(i,:,j)=eforcRBCINF(i,:,j).^2;
            sesHABITINF(i,:,j)=eforcHABITINF(i,:,j).^2;
            sesISHOCKINF(i,:,j)=eforcISHOCKINF(i,:,j).^2;
            sesSTICKYINF(i,:,j)=eforcSTICKYINF(i,:,j).^2;
            sesSTICKYACCINF(i,:,j)=eforcSTICKYACCINF(i,:,j).^2;
            sesUNRES(i,:,j)=eforcUNRES(i,:,j).^2;
            sesCOINT(i,:,j)=eforcCOINT(i,:,j).^2;
            sesLIT(i,:,j)=eforcLIT(i,:,j).^2;
        end
    end
end
toc

%% -------------------------------------------------------------------------
% Root MSES
% -------------------------------------------------------------------------
UNRESrmses=reshape(mean(sesUNRES,1),n,maxhor).^0.5;
RBCrmses025=(reshape(mean(sesRBC(:,:,:,1),1),n,maxhor).^0.5)./UNRESrmses;
HABITrmses025=(reshape(mean(sesHABIT(:,:,:,1),1),n,maxhor).^0.5)./UNRESrmses;
ISHOCKrmses025=(reshape(mean(sesISHOCK(:,:,:,1),1),n,maxhor).^0.5)./UNRESrmses;
STICKYrmses025=(reshape(mean(sesSTICKY(:,:,:,1),1),n,maxhor).^0.5)./UNRESrmses;
COMBOrmses025=(reshape(mean(sesCOMBO(:,:,:,1),1),n,maxhor).^0.5)./UNRESrmses;
STICKYACCrmses025=(reshape(mean(sesSTICKYACC(:,:,:,1),1),n,maxhor).^0.5)./UNRESrmses;
RBCrmses05=(reshape(mean(sesRBC(:,:,:,2),1),n,maxhor).^0.5)./UNRESrmses;
HABITrmses05=(reshape(mean(sesHABIT(:,:,:,2),1),n,maxhor).^0.5)./UNRESrmses;
ISHOCKrmses05=(reshape(mean(sesISHOCK(:,:,:,2),1),n,maxhor).^0.5)./UNRESrmses;
STICKYrmses05=(reshape(mean(sesSTICKY(:,:,:,2),1),n,maxhor).^0.5)./UNRESrmses;
COMBOrmses05=(reshape(mean(sesCOMBO(:,:,:,2),1),n,maxhor).^0.5)./UNRESrmses;
STICKYACCrmses05=(reshape(mean(sesSTICKYACC(:,:,:,2),1),n,maxhor).^0.5)./UNRESrmses;
RBCrmses1=(reshape(mean(sesRBC(:,:,:,3),1),n,maxhor).^0.5)./UNRESrmses;
HABITrmses1=(reshape(mean(sesHABIT(:,:,:,3),1),n,maxhor).^0.5)./UNRESrmses;
ISHOCKrmses1=(reshape(mean(sesISHOCK(:,:,:,3),1),n,maxhor).^0.5)./UNRESrmses;
STICKYrmses1=(reshape(mean(sesSTICKY(:,:,:,3),1),n,maxhor).^0.5)./UNRESrmses;
COMBOrmses1=(reshape(mean(sesCOMBO(:,:,:,3),1),n,maxhor).^0.5)./UNRESrmses;
STICKYACCrmses1=(reshape(mean(sesSTICKYACC(:,:,:,3),1),n,maxhor).^0.5)./UNRESrmses;
RBCrmses5=(reshape(mean(sesRBC(:,:,:,4),1),n,maxhor).^0.5)./UNRESrmses;
HABITrmses5=(reshape(mean(sesHABIT(:,:,:,4),1),n,maxhor).^0.5)./UNRESrmses;
ISHOCKrmses5=(reshape(mean(sesISHOCK(:,:,:,4),1),n,maxhor).^0.5)./UNRESrmses;
STICKYrmses5=(reshape(mean(sesSTICKY(:,:,:,4),1),n,maxhor).^0.5)./UNRESrmses;
COMBOrmses5=(reshape(mean(sesCOMBO(:,:,:,4),1),n,maxhor).^0.5)./UNRESrmses;
STICKYACCrmses5=(reshape(mean(sesSTICKYACC(:,:,:,4),1),n,maxhor).^0.5)./UNRESrmses;
RBCrmses10=(reshape(mean(sesRBC(:,:,:,5),1),n,maxhor).^0.5)./UNRESrmses;
HABITrmses10=(reshape(mean(sesHABIT(:,:,:,5),1),n,maxhor).^0.5)./UNRESrmses;
ISHOCKrmses10=(reshape(mean(sesISHOCK(:,:,:,5),1),n,maxhor).^0.5)./UNRESrmses;
STICKYrmses10=(reshape(mean(sesSTICKY(:,:,:,5),1),n,maxhor).^0.5)./UNRESrmses;
COMBOrmses10=(reshape(mean(sesCOMBO(:,:,:,5),1),n,maxhor).^0.5)./UNRESrmses;
STICKYACCrmses10=(reshape(mean(sesSTICKYACC(:,:,:,5),1),n,maxhor).^0.5)./UNRESrmses;
RBCrmses20=(reshape(mean(sesRBC(:,:,:,6),1),n,maxhor).^0.5)./UNRESrmses;
HABITrmses20=(reshape(mean(sesHABIT(:,:,:,6),1),n,maxhor).^0.5)./UNRESrmses;
ISHOCKrmses20=(reshape(mean(sesISHOCK(:,:,:,6),1),n,maxhor).^0.5)./UNRESrmses;
STICKYrmses20=(reshape(mean(sesSTICKY(:,:,:,6),1),n,maxhor).^0.5)./UNRESrmses;
COMBOrmses20=(reshape(mean(sesCOMBO(:,:,:,6),1),n,maxhor).^0.5)./UNRESrmses;
STICKYACCrmses20=(reshape(mean(sesSTICKYACC(:,:,:,6),1),n,maxhor).^0.5)./UNRESrmses;
COINTrmses=(reshape(mean(sesCOINT,1),n,maxhor).^0.5)./UNRESrmses;
LITrmses=(reshape(mean(sesLIT,1),n,maxhor).^0.5)./UNRESrmses;
RBCrmsesINF=(reshape(mean(sesRBCINF,1),n,maxhor).^0.5)./UNRESrmses;
HABITrmsesINF=(reshape(mean(sesHABITINF,1),n,maxhor).^0.5)./UNRESrmses;
ISHOCKrmsesINF=(reshape(mean(sesISHOCKINF,1),n,maxhor).^0.5)./UNRESrmses;
STICKYrmsesINF=(reshape(mean(sesSTICKYINF,1),n,maxhor).^0.5)./UNRESrmses;
STICKYACCrmsesINF=(reshape(mean(sesSTICKYACCINF,1),n,maxhor).^0.5)./UNRESrmses;

%% Writing Results to csv files

Results025=[RBCrmses025; HABITrmses025;ISHOCKrmses025;STICKYrmses025;STICKYACCrmses025];
Results05=[RBCrmses05; HABITrmses05;ISHOCKrmses05;STICKYrmses05;STICKYACCrmses05];
Results1=[RBCrmses1; HABITrmses1;ISHOCKrmses1;STICKYrmses1;STICKYACCrmses1];
Results5=[RBCrmses5; HABITrmses5;ISHOCKrmses5;STICKYrmses5;STICKYACCrmses5];
Results10=[RBCrmses10; HABITrmses10;ISHOCKrmses10;STICKYrmses10;STICKYACCrmses10];
Results20=[RBCrmses20; HABITrmses20;ISHOCKrmses20;STICKYrmses20;STICKYACCrmses20];
ResultsCOMBO=[COMBOrmses025;COMBOrmses05;COMBOrmses1; COMBOrmses5;COMBOrmses10;COMBOrmses20];
%csvwrite('RBCrmsesINF.csv',RBCrmsesINF);
%csvwrite('HABITrmsesINF.csv',HABITrmsesINF);
%csvwrite('ISHOCKrmsesINF.csv',ISHOCKrmsesINF);
%csvwrite('STICKYrmsesINF.csv',STICKYrmsesINF);
%csvwrite('STICKYACCrmsesINF.csv',STICKYACCrmsesINF);
%csvwrite('Results025.csv',Results025);
%csvwrite('Results05.csv',Results05);
%csvwrite('Results1.csv',Results1);
%csvwrite('Results5.csv',Results5);
%csvwrite('Results10.csv',Results10);
%csvwrite('Results20.csv',Results20);
%csvwrite('UNRESrmses.csv',UNRESrmses);
%csvwrite('LITrmses.csv',LITrmses);
%csvwrite('COINTrmses.csv',COINTrmses);
%csvwrite('ResultsCOMBO.csv',ResultsCOMBO);